from collectd_rest.rrd import render
from django.utils.cache import patch_cache_control
from rest_framework import renderers, status

class ImageRenderer(renderers.BaseRenderer):
	charset = None
	render_style = 'binary'

	def render(self, data, media_type=None, renderer_context=None):
		if renderer_context is None:
			return None

		response = renderer_context['response']
		status_code = renderer_context['response'].status_code
		if not status.is_success(status_code):
			return None

		patch_cache_control(response, max_age=data['max_age'], must_revalidate=True)

		return render(data['command'], self.format)

class PNGRenderer(ImageRenderer):
	media_type = 'image/png'
	format = 'png'

class SVGRenderer(ImageRenderer):
	media_type = 'image/svg+xml'
	format = 'svg'



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-01-18 03:29
from __future__ import unicode_literals

import collectd_rest.models
from django.db import migrations, models
import django.db.models.deletion

def forward_default_graph_granularity(apps, schema_editor):
    GraphGranularity = apps.get_model("collectd_rest", "GraphGranularity")
    db_alias = schema_editor.connection.alias
    objects = GraphGranularity.objects.using(db_alias)
    objects.create(name='default', max_age=0)

def reverse_default_graph_granularity(apps, schema_editor):
    GraphGranularity = apps.get_model("collectd_rest", "GraphGranularity")
    db_alias = schema_editor.connection.alias
    objects = GraphGranularity.objects.using(db_alias)
    objects.get(name='default').delete()

class Migration(migrations.Migration):

    dependencies = [
        ('collectd_rest', '0002_slug_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='GraphGranularity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(max_length=256, unique=True)),
                ('max_age', models.PositiveIntegerField()),
            ],
        ),
        # collectd_rest.models.default_graph_granularity implies that object
        # with name='default' exists
        migrations.RunPython(
            forward_default_graph_granularity,
            reverse_default_graph_granularity
        ),
        migrations.AddField(
            model_name='graph',
            name='granularity',
            field=models.ForeignKey(default=collectd_rest.models.default_graph_granularity, on_delete=django.db.models.deletion.SET_DEFAULT, related_name='graphs', to='collectd_rest.GraphGranularity'),
        ),
    ]



try:
	from django.urls import reverse
except ImportError:
	from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.test import TestCase
from rest_framework import status
from rest_framework.test import APIClient, force_authenticate
from collectd_rest.models import Graph, GraphGroup, GraphGranularity
from collectd_rest.rrd import RRDError
try:
	from unittest.mock import patch, create_autospec
except ImportError:
	from mock import patch, create_autospec

class GraphTest(TestCase):
	def setUp(self):
		self.client = APIClient()

	@patch('collectd_rest.serializers.render')
	def test_graph_create1(self, mock):
		command = 'TEST'
		format = 'PNG'

		url = reverse('graph-list')
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		granularity = GraphGranularity.objects.get(name='default')
		response = self.client.post(url, {
			'title': 'New Graph',
			'name': 'graph1',
			'group': group.name,
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		graph = Graph.objects.get(name='graph1')
		self.assertEqual(graph.group, group)
		self.assertEqual(graph.granularity, granularity)
		mock.assert_called_with(command, format)
	@patch('collectd_rest.serializers.render')
	def test_graph_create2(self, mock):
		command = 'TEST'
		format = 'png'

		url = reverse('graph-list')
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		# name is missed here
		response = self.client.post(url, {
			'title': 'New Graph',
			'group': group.name,
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
		# title may be missed here
		response = self.client.post(url, {
			'name': 'graph2',
			'group': group.name,
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		graph = Graph.objects.get(name='graph2')
		self.assertEqual(graph.title, "")
		# group is missed here
		response = self.client.post(url, {
			'title': 'New Graph',
			'name': 'graph3',
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
	def test_graph_create3(self):
		url = reverse('graph-list')
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		# command is missed here
		response = self.client.post(url, {
			'title': 'New Graph',
			'name': 'graph1',
			'group': group.name}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
	@patch('collectd_rest.serializers.render')
	def test_graph_create4(self, mock):
		command = 'TEST'
		format = 'PNG'

		url = reverse('graph-list')
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		granularity = GraphGranularity.objects.create(name="custom", max_age=42)
		response = self.client.post(url, {
			'title': 'New Graph',
			'name': 'graph1',
			'group': group.name,
			'granularity' : 'custom',
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		graph = Graph.objects.get(name='graph1')
		self.assertEqual(graph.granularity, granularity)
		mock.assert_called_with(command, format)

	def test_graph_detail1(self):
		command = "TEST"
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		granularity = GraphGranularity.objects.get(name='default')
		graph = Graph.objects.create(name="graph1", title="Graph 1", command=command, group=group, granularity=granularity)

		url = reverse('graph-detail', args=[graph.id])
		response = self.client.get(url)
		self.assertEqual(response.status_code, status.HTTP_200_OK)
		json = response.json()
		self.assertEqual(json['id'], graph.id)
		self.assertNotIn("command", json)

	def test_graph_detail2(self):
		command = "TEST"
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		user = User.objects.create(username='gendalf')
		granularity = GraphGranularity.objects.get(name='default')
		graph = Graph.objects.create(name="graph1", title="Graph 1", command=command, group=group, granularity=granularity)

		url = reverse('graph-detail', args=[graph.id])
		self.client.force_authenticate(user=user)
		response = self.client.get(url)
		self.assertEqual(response.status_code, status.HTTP_200_OK)
		json = response.json()
		self.assertEqual(json['id'], graph.id)
		self.assertEqual(json['command'], command)

	@patch('collectd_rest.serializers.render')
	def test_graph_create_duplicates(self, mock):
		command = 'TEST'
		format = 'png'

		url = reverse('graph-list')
		group1 = GraphGroup.objects.create(name="group1", title="Group 1")
		group2 = GraphGroup.objects.create(name="group2", title="Group 2")
		response = self.client.post(url, {
			'title': 'New Graph',
			'name': 'graph',
			'group': group1.name,
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		response = self.client.post(url, {
			'title': 'New Graph',
			'name': 'graph',
			'group': group1.name,
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
		response = self.client.post(url, {
			'title': 'New Graph',
			'name': 'graph',
			'group': group2.name,
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		self.assertEqual(len(Graph.objects.filter(name='graph')),2)

	def graph_render_helper(self, mock, mime, format, max_age):
		command = 'TEST'

		granularity = GraphGranularity.objects.create(name="custom", max_age=max_age)
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		graph = Graph.objects.create(name="graph1", title="Graph 1", command=command, group=group, granularity=granularity)
		url = reverse('graph-detail', args=[graph.id])
		response = self.client.get(url, HTTP_ACCEPT=mime)
		cache_control = [x.strip() for x in response['Cache-Control'].split(",")]
		self.assertEqual(response.status_code, status.HTTP_200_OK)
		self.assertEqual(response["Content-Type"], mime)
		self.assertEqual(sorted(cache_control), ["max-age={0}".format(max_age), 'must-revalidate'])
		mock.assert_called_with(command, format)

	@patch('collectd_rest.renderers.render')
	def test_graph_render_png(self, mock):
		format = 'png'
		mime = "image/png"
		self.graph_render_helper(mock, mime, format, 0)

	@patch('collectd_rest.renderers.render')
	def test_graph_render_svg1(self, mock):
		format = 'svg'
		mime = "image/svg+xml"
		self.graph_render_helper(mock, mime, format, 0)

	@patch('collectd_rest.renderers.render')
	def test_graph_render_svg2(self, mock):
		format = 'svg'
		mime = "image/svg+xml"
		self.graph_render_helper(mock, mime, format, 42)

	@patch('collectd_rest.serializers.render')
	def test_graph_validate1(self, mock):
		mock.side_effect = RRDError('Boom!')
		command = "wrong command"

		url = reverse('graph-list')
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		response = self.client.post(url, {
			'title': 'New Graph',
			'name': 'graph1',
			'group': group.name,
			'command': command}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
		self.assertEqual(response.content,b"{\"command\":[\"Boom!\"]}")

	def test_missing_graph_404_detail(self):
		url = reverse('graph-detail', args=[404])
		response = self.client.get(url)
		self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

	def test_missing_graph_404_png(self):
		url = reverse('graph-detail', args=[404])
		response = self.client.get(url, HTTP_ACCEPT="image/png")
		self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



from collectd_rest import models, serializers, renderers
from rest_framework import viewsets
from rest_framework.settings import api_settings
from django.contrib.auth.models import AnonymousUser

class GraphGranularityViewSet(viewsets.ModelViewSet):
	queryset = models.GraphGranularity.objects.all()
	serializer_class = serializers.GraphGranularitySerializer
	lookup_field = 'name'

class GraphGroupViewSet(viewsets.ModelViewSet):
	queryset = models.GraphGroup.objects.prefetch_related('graphs', 'graphs__granularity')
	serializer_class = serializers.GraphGroupSerializer
	lookup_field = 'name'

class GraphViewSet(viewsets.ModelViewSet):
	queryset = models.Graph.objects.select_related('granularity')
	serializer_class = serializers.GraphSerializer
	unauthenticated_serializer_class = serializers.UnauthenticatedGraphSerializer
	renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES + [renderers.PNGRenderer, renderers.SVGRenderer, ]

	def get_serializer_class(self):
		if (not (self.action == "retrieve" or self.action == "list")
			or isinstance(self.request.accepted_renderer, renderers.ImageRenderer)
			or not isinstance(self.request.user, AnonymousUser)):

			return super(GraphViewSet, self).get_serializer_class()

		return self.unauthenticated_serializer_class



from collectd_rest import views
from rest_framework.routers import DefaultRouter
import django

router = DefaultRouter()
router.register(r'granularities', views.GraphGranularityViewSet)
router.register(r'groups', views.GraphGroupViewSet)
router.register(r'graphs', views.GraphViewSet)

if django.VERSION[0] > 1:
	from django.urls import include, re_path
else:
	from django.conf.urls import include, url as re_path

urlpatterns = [
	re_path(r'^', include(router.urls)),
]



import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = 'fake-key'
INSTALLED_APPS = [
	'django.contrib.contenttypes',
	'django.contrib.auth',
	'rest_framework',
	'collectd_rest',
]
REST_FRAMEWORK = {
	'DEFAULT_RENDERER_CLASSES': (
		'rest_framework.renderers.JSONRenderer',
	),
	'DEFAULT_PARSER_CLASSES': (
		'rest_framework.parsers.JSONParser',
	),
}
DATABASES = {
	'default': {
		'ENGINE': 'django.db.backends.sqlite3',
		'NAME': 'test.sqlite3',
	}
}
MIDDLEWARE_CLASSES = []
ROOT_URLCONF = 'tests.urls'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-04-02 11:05
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Graph',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=256)),
                ('title', models.CharField(blank=True, max_length=256)),
                ('priority', models.IntegerField(default=0)),
                ('command', models.TextField()),
            ],
            options={
                'ordering': ['-priority'],
            },
        ),
        migrations.CreateModel(
            name='GraphGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=256, unique=True)),
                ('title', models.CharField(max_length=256)),
            ],
        ),
        migrations.AddField(
            model_name='graph',
            name='group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='graphs', to='collectd_rest.GraphGroup'),
        ),
        migrations.AlterUniqueTogether(
            name='graph',
            unique_together=set([('name', 'group')]),
        ),
    ]



import django

if django.VERSION[0] > 1:
	from django.urls import include, re_path
else:
	from django.conf.urls import include, url as re_path

urlpatterns = [
	re_path(r'', include('collectd_rest.urls')),
]



from collectd_rest import rrd
from django.test import TestCase

class RRDTest(TestCase):
	def test_split1(self):
		ret = rrd.split("abc")
		self.assertListEqual([b'abc',], ret)
	def test_split2(self):
		ret = rrd.split("do\"not\"separate")
		self.assertListEqual([b'donotseparate',], ret)
	def test_split3(self):
		ret = rrd.split("do'not'separate")
		self.assertListEqual([b'donotseparate',], ret)
	def test_split4(self):
		ret = rrd.split("\"\"")
		self.assertListEqual([b'',], ret)
	def test_split5(self):
		ret = rrd.split("\"a b c\"")
		self.assertListEqual([b'a b c',], ret)
	def test_split6(self):
		ret = rrd.split("\"a \\\" c\"")
		self.assertListEqual([b"a \" c",], ret)
	def test_split7(self):
		ret = rrd.split("\"a \\\\ c\"")
		self.assertListEqual([b"a \\ c",], ret)
	def test_split8(self):
		# The escape characters retain its special meaning only when
		# followed by the quote in use, or the escape character itself.
		# Otherwise the escape character will be considered a normal
		# character.
		ret = rrd.split("\"a \\b c\"")
		self.assertListEqual([b"a \\b c",], ret)
		ret = rrd.split("'a \\\" c'")
		self.assertListEqual([b'a \\\" c'], ret)
		ret = rrd.split("'a \\\' c'")
		self.assertListEqual([b'a \' c'], ret)
	def test_split9(self):
		ret = rrd.split("\\b")
		self.assertListEqual([b'b'], ret)
	def test_split10(self):
		ret = rrd.split("a b c")
		self.assertListEqual([b'a', b'b', b'c'], ret)
	def test_split11(self):
		ret = rrd.split("\"a 'b' c\"")
		self.assertListEqual([b'a \'b\' c'], ret)
	def test_split12(self):
		ret = rrd.split("'a \"b\" c'")
		self.assertListEqual([b'a "b" c'], ret)



try:
	from django.urls import reverse
except ImportError:
	from django.core.urlresolvers import reverse
from django.test import TestCase
from rest_framework import status
from rest_framework.test import APIClient
from collectd_rest.models import Graph, GraphGroup, GraphGranularity

class GraphGroupTest(TestCase):
	def setUp(self):
		self.client = APIClient()
	
	def test_group_create1(self):
		url = reverse('graphgroup-list')
		response = self.client.post(url, {
			'title': "The Group",
			'name': 'group1'}, format='json')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		group1 = GraphGroup.objects.get(name="group1")
	def test_group_create2(self):
		url = reverse('graphgroup-list')
		# title is missed here
		response = self.client.post(url, {
			'name': 'group2'}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
		# name is missed here
		response = self.client.post(url, {
			'title': 'The Group'}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
	def test_group_list1(self):
		group = GraphGroup.objects.create(name="group1", title="Group 1")
		granularity = GraphGranularity.objects.get(name='default')
		graph = Graph.objects.create(name="graph1", title="Graph 1", command="format", group=group, granularity=granularity)

		url = reverse('graphgroup-list')
		response = self.client.get(url)
		self.assertEqual(response.status_code, status.HTTP_200_OK)
		json = response.json()
		self.assertEqual(len(json), 1)
